/**
 * Per gestire la stampa nel caso l'utente inserisca il valore 0,
 * introduciamo due funzioni:
 * void stampa_aux(int) che stampa ricorsivamente valori strettamente positivi
 * void stampa(int) che verifica se il valore inserito e' 0 e in tal caso 
 *   lo stampa direttamente, altrimenti (se il valore inserito e' strettamente 
 *   positivo) lo passa alla funzione stampa_aux.
 *
 * Idea per la ricorsione.
 * Caso base: se il valore e' composto da una cifra, stampa direttamente la cifra
 * Passo induttivo: se il valore ha piu' di due cifre (es. 1234), prima stampa 
 *   ricorsivamente le prime cifre (es. 123) e poi stampa l'ultima (es. 4)
 * 
 * n % 10 isola l'ultima cifra di n
 * n / 10 isola le restanti cifre
 */

#include <stdio.h>

/* Forza l'utente a inserire un valore positivo
 */
int leggi(void) {
	int numero;

	do {
		printf("inserisci un numero positivo: ");
		scanf("%d", &numero);
	} while (numero < 0);

	return numero;
}

/* Converte una cifra numerica in parola
 */
void stampa_cifra(int a) {
	switch(a) {
		case 0: 
			printf("zero ");
			break;
		case 1: 
			printf("uno ");
		break;
		case 2: 
			printf("due ");
			break;
		case 3: 
			printf("tre ");
			break;
		case 4:       
			printf("quattro ");
		break;
		case 5: 
			printf("cinque ");
			break;
		case 6: 
			printf("sei ");
			break;
		case 7: 
			printf("sette ");
			break;
		case 8: 
			printf("otto ");
			break;
		case 9: 
			printf("nove ");
			break;
	}
}

/* Funzione che stampa valori strettamente positivi
 */
void stampa_aux(int numero) {
  if (numero != 0) {
    stampa_aux(numero / 10); // prima stampiamo le altre cifre
    stampa_cifra(numero % 10); // poi stampiamo la cifra meno significativa
  }
}

/* Funzione che stampa valori maggiori o uguali a 0,
 * tratta il caso 0 a parte, 
 * per tutti gli altri casi richiama stampa_aux
 */
void stampa(int numero) {
  // isoliamo il caso 0
  if (numero == 0)
    stampa_cifra(numero);
  else
    stampa_aux(numero);
  printf("\n");
}

int main(void) {
	int numero = leggi();
	stampa(numero);
	return 0;
}
