/**
 * Introduciamo una variabile statica last per ricordare l'ultmo valore considerato.
 *
 * Idea per la ricorsione.
 * Caso Base: se last + 1 e' primo allora ritorniamo last + 1 
 *   e incrementiamo last++ per ricordare l'ultimo valore considerato.
 * Passo induttivo: se last + 1 non e' primo, allora continuiamo ricorsivamente la ricerca;
 *   prima della chiamata ricorsiva incrementiamo last++ per ricordare l'ultimo
 *   valore considerato.
 */

#include <stdio.h>

// ============================================================================

int continua(void);
int next_prime(void);
int is_prime( int );

// ============================================================================

int main(void) {
  int p;

  while (continua()) {
    p = next_prime();
    printf("Numero primo: %i\n", p);
  }
  
  return 0;
}

/* chiede all'utente se vuole continuare col prossimo numero primo
 * o se si vuole fermare
 */
int continua(void) {
  char c;
  printf("Vuoi continuare (s/n): ");
  scanf(" %c", &c);
  return (c == 's') ? 1 : 0;
}

/* Sfruttiamo una variabile statica last che considera l'ultimo valore considerato;
 * ogni volta che chiamiamo la funzione il valore di last viene incrementato di 1.
 * La funzione ritorna il prossimo numero primo dopo last.
 */
int next_prime(){
  static int last; // iniziallizzato a 0 (una volta!)
  
  // considero il valore successivo
  last++; 
  // terminiamo se il nuovo valore di last è primo,
  // altrimenti continuiamo con la ricorsione
  return is_prime(last) ? last : next_prime();
}

// ritorna 0 (falso) se l'argomento non e' primo
// ritorna un valore positivo (vero) se l'argomento è primo
int is_prime(int n) {
	int d;
	
	// verifico tutti i possibili divisori di n
	// itero fino alla radice quadrata di n
	for (d = 2; d * d < n; d++)
		if (n % d == 0)
			return 0;
			
	return n >= 2; // 0 e 1 non sono primi
}
