/**
 * La funzione inverti() sfrutta una funzione ricorsiva int inverti_ricorsivo() 
 * che riceve due parametri: n e pot. 
 * Il parametro "n" rappresenta il valore da invertire.
 * Il parametro "pot" rappresenta la massima potenza di 10 che divide n.
 * 
 * Idea per la ricorsione.
 * Caso base: se il valore e' composto da una sola cifra,
 *   ritorniamo il valore stesso.
 * Passo induttivo: se il valore (n) e' composto da piu' cifre,
 *   isoliamo l'ultima cifra (n % 10) e invertiamo
 *   ricorsivamente le restanti cifre (n / 10). 
 *   Per mettere l'ultima cifra in testa, sfruttiamo il parametro pot
 *   che individua la potenza di 10 corrispondente alle cifre di n.
 *
 *   Esempio: consideriamo n = 1234 e pot = 1000
 *   - isoliamo l'ultima cifra 4
 *   - chiamiamo ricorsivamente su 123 e 100 
 *   - la ricorsione ritorna 321
 *   - per mettere in testa il 4, lo moltiplichiamo per pot e sommiamo.
 *     In pratica ritorniamo 4 * pot + 321 (ovvero 4321) 
 */
  

#include <stdio.h>

int leggi(void);
int inverti (int);
int inverti_ricorsivo (int, int);


int main(void) {
	int m = leggi();
	int n = leggi();
	printf("Il valore di %d invertito e' %d\n", m, inverti(m));
	printf("Il valore di %d invertito e' %d\n", n, inverti(n));
	return 0;
}


/* Questa è la funzione richiesta dall'esercizio.
 * Riceve come unico parametro n e ritorna l'intero corrispondente all'inversione di n. 
 * Chiama la funzione ricorsiva inverti_ricorsivo() passando come argomenti
 * il valore originale di n e la massima potenza di 10 che lo divide.
 */
int inverti (int n) {
	int pot;

	// calcoliamo la massima potenza di 10 che divide n	
	for (pot = 1; pot * 10 < n; pot *= 10);

	// chiamiamo la funzione ricorsiva per avere il risultato
	return inverti_ricorsivo(n, pot);
}


/* Calcola ricorsivamente l'intero corrispondente all'inversione di n.
 * Richiede il parametro ausiliario "pot" che individua la massima
 * potenza di 10 che divide il numero n.
 */
int inverti_ricorsivo (int n, int pot) {
  int ultima_cifra; // ultima cifra di n
  int prima_parte;  // prime cifre di n tranne l'ultima
  int prima_parte_invertita; // valore di prima_parte invertita
  
  if (n < 10) // isoliamo il caso a una cifra
    return n;

  else { // passo ricorsivo: almeno due cifre
    ultima_cifra = n % 10;
    prima_parte = n / 10;
    prima_parte_invertita = inverti_ricorsivo(prima_parte, pot / 10);
    
    return ultima_cifra * pot + prima_parte_invertita;
  }
}


/* Forza l'utente a inserire un valore positivo
 */
int leggi(void) {
	int numero;

	do {
		printf("inserisci un numero positivo: ");
		scanf("%d", &numero);
	} while (numero < 0);

	return numero;
}
